'use strict';

var constant = require('../constant.cjs');
var css = require('@master/css');
var webCustomData = require('vscode-css-languageservice/lib/umd/data/webCustomData');
var dataProvider = require('vscode-css-languageservice/lib/umd/languageFacts/dataProvider');

let cssKeys = [];
cssKeys = cssKeys.concat(constant.masterCssOtherKeys);
constant.masterCssKeyValues.forEach((x)=>{
    cssKeys = cssKeys.concat(x.key);
});
const masterCssKeys = [
    ...new Set(cssKeys)
];
// temporary
function checkConfigColorsBlock(document, position) {
    const lineText = (document?.getText({
        start: {
            line: position.line,
            character: 0
        },
        end: {
            line: position.line,
            character: position.character
        }
    }) ?? '').trim();
    if ((lineText.match(/'|"|`/g)?.length ?? 0) % 2 === 0) {
        return false;
    }
    for(let i = position.line; i > 0; i--){
        const text = document.getText({
            start: {
                line: i - 1,
                character: 0
            },
            end: {
                line: i,
                character: 0
            }
        });
        if (text.includes('}')) {
            return false;
        } else if (text.includes('colors:')) {
            return true;
        }
    }
    return false;
}
function getLastInstance(lineText, position, language) {
    const classPattern = /(?:[^"{'\s])+(?=>\s|\b)/g;
    let classMatch;
    let lastKey = '';
    let triggerKey = lineText.charAt(lineText.length - 1);
    let isStart = position.character == 1 || lineText.charAt(lineText.length - 2) === ' ' || lineText.charAt(lineText.length - 2) === '' || lineText.charAt(lineText.length - 2) === '"' || lineText.charAt(lineText.length - 2) === '\'' || lineText.charAt(lineText.length - 2) === '{';
    if (lineText.match(classPattern) === null) {
        return {
            isInstance: false,
            lastKey: '',
            triggerKey: '',
            isStart: false,
            language: language
        };
    } else {
        while((classMatch = classPattern.exec(lineText)) !== null){
            lastKey = classMatch[0];
        }
    }
    if (lineText.charAt(lineText.length - 2) === ':' && lineText.charAt(lineText.length - 1) === ':') {
        triggerKey = '::';
        isStart = false;
    }
    return {
        isInstance: true,
        lastKey: lastKey,
        triggerKey: triggerKey,
        isStart: isStart,
        language: language
    };
}
function getConfigColorsCompletionItem(css$1 = new css.MasterCSS()) {
    let masterStyleCompletionItem = [];
    masterStyleCompletionItem = masterStyleCompletionItem.concat(getColorsItem(css$1));
    return masterStyleCompletionItem;
}
function getCompletionItem(instance, triggerKey, isStart, language, css$1 = new css.MasterCSS()) {
    const cssDataProvider = new dataProvider.CSSDataProvider(webCustomData.cssData);
    const cssProperties = cssDataProvider.provideProperties();
    let masterStyleCompletionItem = [];
    const haveValue = instance.split(':').length;
    let key = instance.split(':')[0];
    key = key.trim();
    const instanceLength = instance.split(':|@').length;
    const last = instance.split(':|@')[instanceLength - 1];
    const mediaPattern = /[^\\s"]+@+([^\\s:"@]+)/g;
    const elementsPattern = /[^\\s"]+::+([^\\s:"@]+)/g;
    let isColorful = false;
    const isMedia = !(mediaPattern.exec(instance) === null && triggerKey !== '@');
    const isElements = !(elementsPattern.exec(instance) === null && triggerKey !== '::');
    const masterCssKeyCompletionItems = [];
    let masterCssValues = [];
    const masterCustomSelectors = Object.keys(css$1.config?.selectors ?? {}).map((x)=>x.endsWith('(') ? `${x})` : x).filter((x)=>x.match(/:[^:]+/) && !constant.masterCssSelectors.find((existedSelector)=>(typeof existedSelector === 'string' ? existedSelector : existedSelector.label) === `:${x}`)).map((x)=>{
        x = x.substring(1);
        if (x.endsWith(')')) {
            return {
                label: x,
                kind: 3
            };
        }
        return x;
    });
    const masterCustomElements = Object.keys(css$1.config?.selectors ?? {}).map((x)=>x.endsWith('(') ? `${x})` : x).filter((x)=>x.match(/::[^:]+/) && !constant.masterCssElements.find((existedElement)=>(typeof existedElement === 'string' ? existedElement : existedElement.label) === `::${x}`)).map((x)=>{
        x = x.substring(2);
        if (x.endsWith(')')) {
            return {
                label: x,
                kind: 3
            };
        }
        return x;
    });
    constant.masterCssKeyValues.forEach((x)=>{
        const fullKey = x.key[0];
        const originalCssProperty = cssProperties.find((x)=>x.name == fullKey);
        const originalCssValues = originalCssProperty?.values ?? [];
        for (const masterKey of x.key){
            if (!masterCssKeyCompletionItems.find((existedValue)=>existedValue.label === masterKey + ':')) {
                masterCssKeyCompletionItems.push({
                    label: masterKey + ':',
                    kind: 10,
                    documentation: originalCssProperty?.description ?? '',
                    command: {
                        title: 'triggerSuggest',
                        command: 'editor.action.triggerSuggest'
                    }
                });
            }
        }
        if (x.key.includes(key)) {
            // constant.ts custom values
            masterCssValues = masterCssValues.concat(x.values.filter((cssValue)=>!masterCssValues.find((existedValue)=>(typeof existedValue === 'string' ? existedValue : existedValue.label) === (typeof cssValue === 'string' ? cssValue : cssValue.label))));
            masterCssValues = masterCssValues.concat(originalCssValues.filter((cssValue, index)=>cssValue.description || !cssValue.description && originalCssValues.indexOf(cssValue) === index).map((cssValue)=>({
                    label: cssValue.name.replace(/,\s/g, ',').replace(/\s/g, '|').replace(/["']/g, ''),
                    kind: 10,
                    documentation: cssValue?.description ?? ''
                })).filter((cssValue)=>!masterCssValues.find((existedValue)=>(typeof existedValue === 'string' ? existedValue : existedValue.label) === (typeof cssValue === 'string' ? cssValue : cssValue.label))));
            if (css$1.config?.variables?.[fullKey]) {
                const masterCustomVariables = Object.keys(css$1.config?.variables[fullKey]);
                masterCssValues = masterCssValues.concat(masterCustomVariables.filter((customValue)=>!masterCssValues.find((existedValue)=>(typeof existedValue === 'string' ? existedValue : existedValue.label) === customValue)));
            }
            if (x.colored) {
                isColorful = true;
                const needPushList = constant.masterCssType.find((y)=>y.type === 'color')?.values.filter((z)=>!masterCssValues.find((a)=>(typeof a === 'string' ? a : a.label) === (typeof z === 'string' ? z : z.label)));
                if (needPushList) {
                    masterCssValues = masterCssValues.concat(needPushList);
                }
            }
        }
    });
    if ((isStart == true || !masterCssKeys.includes(key)) && triggerKey !== '@' && triggerKey !== ':') {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(masterCssKeyCompletionItems);
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(Object.keys(css$1.config?.semantics ?? {}), 10));
        if (language == 'tsx' || language == 'vue' || language == 'jsx') {
            return HaveDash(key, masterStyleCompletionItem);
        }
        return masterStyleCompletionItem;
    } else if (isStart == true) {
        return [];
    }
    if (masterCssKeys.includes(key) && key !== null && isElements === true) {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(constant.masterCssElements, 10));
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(masterCustomElements, 10));
        if ((language == 'tsx' || language == 'vue' || language == 'jsx') && triggerKey !== '@' && triggerKey !== ':') {
            return HaveDash(last, masterStyleCompletionItem);
        }
        return masterStyleCompletionItem;
    }
    if (masterCssKeys.includes(key) && key !== null && isMedia === true) {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(constant.masterCssMedia, 10));
        // todo: refactor theme
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem([
            'light',
            'dark'
        ], 10));
        if ((language == 'tsx' || language == 'vue' || language == 'jsx') && triggerKey !== '@' && triggerKey !== ':') {
            return HaveDash('@' + last, masterStyleCompletionItem);
        }
        return masterStyleCompletionItem;
    }
    if (Object.keys(css$1.config?.semantics ?? {}).includes(key) && !constant.masterCssKeyValues.find((x)=>x.key.includes(key))) {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(constant.masterCssSelectors, 10));
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(masterCustomSelectors, 10));
        if ((language == 'tsx' || language == 'vue' || language == 'jsx') && triggerKey !== '@' && triggerKey !== ':') {
            return HaveDash(last, masterStyleCompletionItem);
        }
        return masterStyleCompletionItem;
    } else if (masterCssKeys.includes(key) && haveValue <= 2 && !(haveValue == 2 && triggerKey === ':')) {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(masterCssValues, 10));
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(constant.masterCssCommonValues, 13).map((x)=>{
            x.sortText = 'z' + x;
            return x;
        }));
        if (isColorful) {
            masterStyleCompletionItem = masterStyleCompletionItem.concat(getColorsItem(css$1));
        }
        if ((language == 'tsx' || language == 'vue' || language == 'jsx') && triggerKey !== '@' && triggerKey !== ':') {
            return HaveDash(last, masterStyleCompletionItem);
        }
        return masterStyleCompletionItem;
    }
    if (masterCssKeys.includes(key) && (haveValue == 2 && triggerKey === ':' || haveValue >= 3) || constant.masterCssKeyValues.find((x)=>x.values.includes(key))) {
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(constant.masterCssSelectors, 3));
        masterStyleCompletionItem = masterStyleCompletionItem.concat(getReturnItem(masterCustomSelectors, 10));
    }
    if ((language == 'tsx' || language == 'vue' || language == 'jsx') && triggerKey !== '@' && triggerKey !== ':') {
        return HaveDash(last, masterStyleCompletionItem);
    }
    return masterStyleCompletionItem;
}
function getReturnItem(items, kind, insertText = '') {
    const masterStyleCompletionItem = [];
    items.forEach((x)=>{
        if (typeof x === 'string') {
            masterStyleCompletionItem.push({
                label: x + insertText,
                kind: kind,
                insertText: x + insertText,
                insertTextMode: 2,
                command: insertText === ':' ? {
                    title: 'triggerSuggest',
                    command: 'editor.action.triggerSuggest'
                } : undefined
            });
        } else {
            masterStyleCompletionItem.push({
                insertText: x.label + insertText,
                insertTextMode: 2,
                command: insertText === ':' ? {
                    title: 'triggerSuggest',
                    command: 'editor.action.triggerSuggest'
                } : undefined,
                ...x
            });
        }
    });
    return masterStyleCompletionItem;
}
function getColorsItem(css$1 = new css.MasterCSS()) {
    const masterStyleCompletionItem = [];
    for(const eachVariableName in css$1.variables){
        const eachVariable = css$1.variables[eachVariableName];
        if (eachVariable.type === 'color') {
            let colorValue;
            let colorSpace;
            if (eachVariable.themes) {
                colorValue = Object.values(eachVariable.themes)[0].value;
                colorSpace = Object.values(eachVariable.themes)[0].space;
            } else {
                colorValue = eachVariable.value;
                colorSpace = eachVariable.space;
            }
            let colorResult;
            switch(colorSpace){
                case 'rgb':
                    colorResult = `rgb(${colorValue?.split(' ')?.slice(0, 3).join(',')})`;
                    break;
                case 'hsl':
                    colorResult = `hsl(${colorValue?.split(' ')?.slice(0, 3).join(',')})`;
                    break;
                case 'hex':
                    colorResult = `#${colorValue}`;
                    break;
            }
            masterStyleCompletionItem.push({
                label: eachVariableName,
                documentation: colorResult,
                kind: 16,
                sortText: eachVariableName
            });
        }
    }
    return masterStyleCompletionItem;
}
function HaveDash(str, itemList) {
    const completionItem = [];
    if (str.split('-').length - 1 <= 0) {
        return itemList;
    } else {
        const start = str.split('-')[0] + '-';
        itemList.map((x)=>{
            if (x.label.includes(start)) {
                completionItem.push({
                    label: x.label,
                    kind: x.kind,
                    insertText: x.insertText?.substring(start.length),
                    insertTextMode: x.insertTextMode,
                    command: x.command
                });
            }
        });
        return completionItem;
    }
}

exports.checkConfigColorsBlock = checkConfigColorsBlock;
exports.getCompletionItem = getCompletionItem;
exports.getConfigColorsCompletionItem = getConfigColorsCompletionItem;
exports.getLastInstance = getLastInstance;
exports.getReturnItem = getReturnItem;
