'use strict';

var css = require('@master/css');
var getCssEntryMarkdownDescription = require('../utils/get-css-entry-markdown-description.cjs');
var beautifyCss = require('js-beautify/js/lib/beautify-css');
var webCustomData = require('vscode-css-languageservice/lib/umd/data/webCustomData');
var dataProvider = require('vscode-css-languageservice/lib/umd/languageFacts/dataProvider');

function doHover(instance, range, config) {
    const css$1 = new css.MasterCSS(config);
    css$1.add(instance);
    const contents = [];
    if (css$1.text) {
        const cssPreview = getCssPreview(css$1.text);
        if (cssPreview) {
            contents.push(cssPreview);
        }
    }
    if (css$1.rules.length > 0) {
        const cssHoverInfo = getCssHoverInfo(css$1.rules[0]);
        if (cssHoverInfo) {
            contents.push(cssHoverInfo);
        }
    }
    return {
        contents,
        range: range
    };
}
function getCssPreview(renderText) {
    return {
        language: 'css',
        value: beautifyCss.css_beautify(renderText, {
            newline_between_rules: true,
            end_with_newline: true
        })
    };
}
function getCssHoverInfo(rule) {
    const cssDataProvider = new dataProvider.CSSDataProvider(webCustomData.cssData);
    const cssProperties = cssDataProvider.provideProperties();
    let cssHoverInfo = null;
    const fullKey = rule.id;
    const originalCssProperty = cssProperties.find((x)=>x.name == fullKey);
    if (rule?.layer && [
        css.Layer.Core,
        css.Layer.CoreNative,
        css.Layer.CoreShorthand,
        css.Layer.CoreNativeShorthand
    ].includes(rule.layer) && originalCssProperty) {
        if (!originalCssProperty.references?.find((x)=>x.name === 'Master Reference')) {
            originalCssProperty.references = [
                ...originalCssProperty?.references ?? [],
                {
                    name: 'Master Reference',
                    url: `https://rc.css.master.co/docs/${fullKey}`
                }
            ];
        }
        cssHoverInfo = getCssEntryMarkdownDescription.getCssEntryMarkdownDescription(originalCssProperty);
    }
    return cssHoverInfo;
}

exports.doHover = doHover;
