import { MasterCSS, Layer } from '@master/css';
import { getCssEntryMarkdownDescription } from '../utils/get-css-entry-markdown-description.mjs';
import { css_beautify } from 'js-beautify/js/lib/beautify-css';
import { cssData } from 'vscode-css-languageservice/lib/umd/data/webCustomData';
import { CSSDataProvider } from 'vscode-css-languageservice/lib/umd/languageFacts/dataProvider';

function doHover(instance, range, config) {
    const css = new MasterCSS(config);
    css.add(instance);
    const contents = [];
    if (css.text) {
        const cssPreview = getCssPreview(css.text);
        if (cssPreview) {
            contents.push(cssPreview);
        }
    }
    if (css.rules.length > 0) {
        const cssHoverInfo = getCssHoverInfo(css.rules[0]);
        if (cssHoverInfo) {
            contents.push(cssHoverInfo);
        }
    }
    return {
        contents,
        range: range
    };
}
function getCssPreview(renderText) {
    return {
        language: 'css',
        value: css_beautify(renderText, {
            newline_between_rules: true,
            end_with_newline: true
        })
    };
}
function getCssHoverInfo(rule) {
    const cssDataProvider = new CSSDataProvider(cssData);
    const cssProperties = cssDataProvider.provideProperties();
    let cssHoverInfo = null;
    const fullKey = rule.id;
    const originalCssProperty = cssProperties.find((x)=>x.name == fullKey);
    if (rule?.layer && [
        Layer.Core,
        Layer.CoreNative,
        Layer.CoreShorthand,
        Layer.CoreNativeShorthand
    ].includes(rule.layer) && originalCssProperty) {
        if (!originalCssProperty.references?.find((x)=>x.name === 'Master Reference')) {
            originalCssProperty.references = [
                ...originalCssProperty?.references ?? [],
                {
                    name: 'Master Reference',
                    url: `https://rc.css.master.co/docs/${fullKey}`
                }
            ];
        }
        cssHoverInfo = getCssEntryMarkdownDescription(originalCssProperty);
    }
    return cssHoverInfo;
}

export { doHover };
